import json

from django.shortcuts import render_to_response
from django.template import RequestContext
from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.models import User

import ox
from ox.django.api import actions
from ox.django.decorators import login_required_json
from ox.django.shortcuts import json_response, render_to_json_response, get_object_or_404_json

from django_lsd.models import Item

def index(request):
    context = RequestContext(request, {
    })
    return render_to_response('index.html', context)

def init(request):
    response = json_response({})
    if request.user.is_authenticated():
        response['data']['user'] = request.user.username
    return render_to_json_response(response)
actions.register(init)

def signin(request):
    data = json.loads(request.POST['data'])
    if 'username' in data and 'password' in data:
        data['username'] = data['username'].strip()
        qs = User.objects.filter(username__iexact=data['username'])
        if qs.count() == 0:
            response = json_response({
                'errors': {
                    'username': 'Unknown Username'
                }
            })
            username = None
        else:
            username = qs[0].username
        if username:
            user = authenticate(username=username, password=data['password'])
            if user is not None:
                if user.is_active:
                    request.session['ui'] = '{}'
                    login(request, user)
                    response = json_response({
                        'user': user.username
                    })
                else:
                    response = json_response({
                        'errors': {
                            'username': 'User Disabled'
                        }
                    })
            else:
                response = json_response({
                    'errors': {
                        'password': 'Incorrect Password'
                    }
                })
    else:
        response = json_response(status=400, text='invalid data')
    return render_to_json_response(response)
actions.register(signin, cache=False)

def signout(request):
    response = json_response(text='ok')
    if request.user.is_authenticated():
        response = json_response(text='logged out')
        logout(request)
    return render_to_json_response(response)
actions.register(signout, cache=False)

def signup(request):
    data = json.loads(request.POST['data'])
    if 'username' in data and 'password' in data:
        data['username'] = data['username'].strip()
        if 'email' in data:
            data['email'] = ox.escape_html(data['email'])
        if User.objects.filter(username__iexact=data['username']).count() > 0:
            response = json_response({
                'errors': {
                    'username': 'Username already exists'
                }
            })
        elif User.objects.filter(email__iexact=data['email']).count() > 0:
            response = json_response({
                'errors': {
                    'email': 'Email address already exists'
                }
            })
        elif not data['password']:
            response = json_response({
                'errors': {
                    'password': 'Password can not be empty'
                }
            })
        else:
            first_user = User.objects.count() == 0
            user = User(username=data['username'], email=data['email'])
            user.set_password(data['password'])
            #make first user admin
            user.is_superuser = first_user
            user.is_staff = first_user
            user.save()
            user = authenticate(username=data['username'], password=data['password'])
            login(request, user)
            response = json_response({
            }, text='account created')
    else:
        response = json_response(status=400, text='invalid data')
    return render_to_json_response(response)
actions.register(signup, cache=False)



# file views
@login_required_json
def find(request):
    response = json_response()
    response['data']['items'] = []
    for i in Item.objects.all():
        j = {}
        j.update(i.data)
        j['id'] = i.sha1
        j['sha1'] = i.sha1
        j['info_hash'] = i.info_hash
        if i.lsd_files.filter(user=request.user).exists():
            state = 'available'
        else:
            state = ''
        j['state'] = state
        response['data']['items'].append(j)
    return render_to_json_response(response)
actions.register(find, cache=True)

@login_required_json
def edit(request):
    response = json_response()
    data = json.loads(request.POST['data'])
    sha1 = data.pop('id')
    item = get_object_or_404_json(Item, sha1=sha1)
    for key in data:
        if key not in ('id', 'sha1', 'info_hash'):
            item.data[key] = data[key]
    item.save()
    return render_to_json_response(response)
actions.register(find, cache=True)
